@@ -28,9 +28,7 @@ class GPfluxPredictor(SupportsGetObservationNoise, ABC):
     """
     A trainable wrapper for a GPflux deep Gaussian process model. The code assumes subclasses
     will use the Keras `fit` method for training, and so they should provide access to both a
-    `model_keras` and `model_gpflux`. Note: due to Keras integration, the user should remember to
-    use `tf.keras.backend.set_floatx()` with the desired value (consistent with GPflow) to avoid
-    dtype errors.
+    `model_keras` and `model_gpflux`.
     """
 
     def __init__(self, optimizer: KerasOptimizer | None = None):
