@@ -14,16 +14,18 @@
 
 from __future__ import annotations
 
-from typing import Any, Callable
+from typing import Any, Callable, Optional
 
 import dill
 import gpflow
 import tensorflow as tf
 from gpflow.inducing_variables import InducingPoints
+from gpflow.utilities.traversal import _merge_leaf_components, leaf_components
 from gpflux.layers import GPLayer, LatentVariableLayer
 from gpflux.models import DeepGP
 from tensorflow.python.keras.callbacks import Callback
 
+from ... import logging
 from ...data import Dataset
 from ...types import TensorType
 from ..interfaces import (
@@ -48,8 +50,6 @@ class DeepGaussianProcess(
     A :class:`TrainableProbabilisticModel` wrapper for a GPflux :class:`~gpflux.models.DeepGP` with
     :class:`GPLayer` or :class:`LatentVariableLayer`: this class does not support e.g. keras layers.
     We provide simple architectures that can be used with this class in the `architectures.py` file.
-    Note: the user should remember to set `tf.keras.backend.set_floatx()` with the desired value
-    (consistent with GPflow) so that dtype errors do not occur.
     """
 
     def __init__(
@@ -62,13 +62,12 @@ class DeepGaussianProcess(
         """
         :param model: The underlying GPflux deep Gaussian process model. Passing in a named closure
             rather than a model can help when copying or serialising.
-        :param optimizer: The optimizer configuration for training the model. Defaults to
-            :class:`~trieste.models.optimizer.KerasOptimizer` wrapper with
-            :class:`~tf.optimizers.Adam` optimizer. The ``optimizer`` argument to the wrapper is
-            used when compiling the model and ``fit_args`` is a dictionary of arguments to be used
-            in the Keras ``fit`` method. Defaults to 400 epochs, batch size of 1000, and verbose 0.
-            A custom callback that reduces the optimizer learning rate is used as well. See
-            https://keras.io/api/models/model_training_apis/#fit-method for a list of possible
+        :param optimizer: The optimizer wrapper with necessary specifications for compiling and
+            training the model. Defaults to :class:`~trieste.models.optimizer.KerasOptimizer` with
+            :class:`~tf.optimizers.Adam` optimizer, mean squared error metric and a dictionary of
+            default arguments for the Keras `fit` method: 400 epochs, batch size of 1000, and
+            verbose 0. A custom callback that reduces the optimizer learning rate is used as well.
+            See https://keras.io/api/models/model_training_apis/#fit-method for a list of possible
             arguments.
         :param num_rff_features: The number of random Fourier features used to approximate the
             kernel when calling :meth:`trajectory_sampler`. We use a default of 1000 as it typically
@@ -126,10 +125,25 @@ class DeepGaussianProcess(
                 "callbacks": [tf.keras.callbacks.LearningRateScheduler(scheduler)],
             }
 
-        self._model_gpflux = model
+        if self.optimizer.metrics is None:
+            self.optimizer.metrics = ["mse"]
 
+        self._model_gpflux = model
+        # inputs and targets need to be redone with a float64 dtype to avoid setting the keras
+        # backend to float64, this is likely to be fixed in GPflux, see issue:
+        # https://github.com/secondmind-labs/GPflux/issues/76
+        self._model_gpflux.inputs = tf.keras.Input(
+            tuple(self._model_gpflux.inputs.shape[:-1]),
+            name=self._model_gpflux.inputs.name,
+            dtype=tf.float64,
+        )
+        self._model_gpflux.targets = tf.keras.Input(
+            tuple(self._model_gpflux.targets.shape[:-1]),
+            name=self._model_gpflux.targets.name,
+            dtype=tf.float64,
+        )
         self._model_keras = model.as_training_model()
-        self._model_keras.compile(self.optimizer.optimizer)
+        self._model_keras.compile(self.optimizer.optimizer, metrics=self.optimizer.metrics)
         self._absolute_epochs = 0
         self._continuous_optimisation = continuous_optimisation
 
@@ -141,46 +155,109 @@ class DeepGaussianProcess(
             state["_model_gpflux"] = gpflow.utilities.parameter_dict(self._model_gpflux)
             state["_model_keras"] = gpflow.utilities.parameter_dict(self._model_keras)
 
-        # serialize all the callback models, pickle the optimizer(!), and revert (ugh!)
+        # use to_json and get_weights to save any optimizer fit_arg callback models
+        callbacks: list[Callback] = self._optimizer.fit_args.get("callbacks", [])
         callback: Callback
         saved_models: list[KerasOptimizer] = []
-        for callback in self._optimizer.fit_args.get("callbacks", []):
-            saved_models.append(callback.model)
-            if callback.model is self._model_keras:
-                # no need to serialize the main model (and it probably wouldn't work anyway)
-                callback.model = ...
-            elif callback.model:
-                callback.model = (callback.model.to_json(), callback.model.get_weights())
-        state["_optimizer"] = dill.dumps(state["_optimizer"])
-        for callback, model in zip(self._optimizer.fit_args.get("callbacks", []), saved_models):
-            callback.model = model
+        tensorboard_writers: list[dict[str, Any]] = []
+        try:
+            for callback in callbacks:
+                # serialize the callback models before pickling the optimizer
+                saved_models.append(callback.model)
+                if callback.model is self._model_keras:
+                    # no need to serialize the main model, just use a special value instead
+                    callback.model = ...
+                elif callback.model:
+                    callback.model = (callback.model.to_json(), callback.model.get_weights())
+                # don't pickle tensorboard writers either; they'll be recreated when needed
+                if isinstance(callback, tf.keras.callbacks.TensorBoard):
+                    tensorboard_writers.append(callback._writers)
+                    callback._writers = {}
+            state["_optimizer"] = dill.dumps(state["_optimizer"])
+        except Exception as e:
+            raise NotImplementedError(
+                "Failed to copy DeepGaussianProcess optimizer due to unsupported callbacks."
+            ) from e
+        finally:
+            # revert original state, even if the pickling failed
+            for callback, model in zip(self._optimizer.fit_args.get("callbacks", []), saved_models):
+                callback.model = model
+            for callback, writers in zip(
+                (cb for cb in callbacks if isinstance(cb, tf.keras.callbacks.TensorBoard)),
+                tensorboard_writers,
+            ):
+                callback._writers = writers
+
+        # do the same thing for the history callback
+        if self._model_keras.history:
+            history_model = self._model_keras.history.model
+            try:
+                if history_model is self._model_keras:
+                    # no need to serialize the main model, just use a special value instead
+                    self._model_keras.history.model = ...
+                elif history_model:
+                    self._model_keras.history.model = (
+                        history_model.to_json(),
+                        history_model.get_weights(),
+                    )
+                state["_history"] = dill.dumps(self._model_keras.history)
+            finally:
+                self._model_keras.history.model = history_model
 
         return state
 
     def __setstate__(self, state: dict[str, Any]) -> None:
         self.__dict__.update(state)
 
+        # regenerate the models using the model closure
         if self._model_closure is not None:
             dgp: DeepGP = state["_model_closure"]()
             self._model_gpflux = dgp
+            # inputs and targets need to be redone with a float64 dtype to avoid setting the keras
+            # backend to float64, this is likely to be fixed in GPflux, see issue:
+            # https://github.com/secondmind-labs/GPflux/issues/76
+            self._model_gpflux.inputs = tf.keras.Input(
+                tuple(self._model_gpflux.inputs.shape[:-1]),
+                name=self._model_gpflux.inputs.name,
+                dtype=tf.float64,
+            )
+            self._model_gpflux.targets = tf.keras.Input(
+                tuple(self._model_gpflux.targets.shape[:-1]),
+                name=self._model_gpflux.targets.name,
+                dtype=tf.float64,
+            )
             self._model_keras = dgp.as_training_model()
 
+        # unpickle the optimizer, and restore all the callback models
         self._optimizer = dill.loads(self._optimizer)
         for callback in self._optimizer.fit_args.get("callbacks", []):
             if callback.model is ...:
-                callback.model = self._model_keras
+                callback.set_model(self._model_keras)
             elif callback.model:
-                model, weights = callback.model
-                callback.model = tf.keras.models.model_from_json(
-                    model,
-                )
-                callback.model.set_weights(weights)
+                model_json, weights = callback.model
+                model = tf.keras.models.model_from_json(model_json)
+                model.set_weights(weights)
+                callback.set_model(model)
+
+        # recompile the model
         self._model_keras.compile(self._optimizer.optimizer)
 
+        # assign the model parameters
         if self._model_closure is not None:
             gpflow.utilities.multiple_assign(self._model_gpflux, state["_model_gpflux"])
             gpflow.utilities.multiple_assign(self._model_keras, state["_model_keras"])
 
+        # restore the history (including any model it contains)
+        if "_history" in state:
+            self._model_keras.history = dill.loads(state["_history"])
+            if self._model_keras.history.model is ...:
+                self._model_keras.history.set_model(self._model_keras)
+            elif self._model_keras.history.model:
+                model_json, weights = self._model_keras.history.model
+                model = tf.keras.models.model_from_json(model_json)
+                model.set_weights(weights)
+                self._model_keras.history.set_model(model)
+
     def __repr__(self) -> str:
         """"""
         return f"DeepGaussianProcess({self.model_gpflux!r}, {self.optimizer.optimizer!r})"
@@ -195,9 +272,9 @@ class DeepGaussianProcess(
 
     def sample(self, query_points: TensorType, num_samples: int) -> TensorType:
         trajectory = self.trajectory_sampler().get_trajectory()
-        expanded_query_points = tf.expand_dims(query_points, -2)
-        tiled_query_points = tf.tile(expanded_query_points, [1, num_samples, 1])
-        return tf.expand_dims(tf.transpose(trajectory(tiled_query_points), [1, 0]), -1)
+        expanded_query_points = tf.expand_dims(query_points, -2)  # [N, 1, D]
+        tiled_query_points = tf.tile(expanded_query_points, [1, num_samples, 1])  # [N, S, D]
+        return tf.transpose(trajectory(tiled_query_points), [1, 0, 2])  # [S, N, L]
 
     def reparam_sampler(self, num_samples: int) -> ReparametrizationSampler[GPfluxPredictor]:
         """
@@ -281,3 +358,71 @@ class DeepGaussianProcess(
         # so that the next time we call `optimize` the starting learning rate would be different.
         # Therefore we make sure the learning rate is set back to its initial value.
         self.optimizer.optimizer.lr.assign(self.original_lr)
+
+    def log(self, dataset: Optional[Dataset] = None) -> None:
+        """
+        Log model training information at a given optimization step to the Tensorboard.
+        We log a few summary statistics of losses, layer KL divergences and metrics (as provided in
+        ``optimizer``): ``final`` value at the end of the training, ``diff`` value as a difference
+        between inital and final epoch. We also log epoch statistics, but as histograms, rather
+        than time series. We also log several training data based metrics, such as root mean square
+        error between predictions and observations and several others.
+
+        For custom logs user will need to subclass the model and overwrite this method.
+
+        :param dataset: Optional data that can be used to log additional data-based model summaries.
+        """
+        summary_writer = logging.get_tensorboard_writer()
+        if summary_writer:
+            with summary_writer.as_default(step=logging.get_step_number()):
+                logging.scalar("epochs/num_epochs", len(self.model_keras.history.epoch))
+                # kernel parameters
+                for idx, layer in enumerate(self.model_gpflux.f_layers):
+                    kernel_components = _merge_leaf_components(leaf_components(layer.kernel))
+                    for k, v in kernel_components.items():
+                        if v.trainable:
+                            if tf.rank(v) == 0:
+                                logging.scalar(f"layer[{idx}]/kernel.{k}", v)
+                            elif tf.rank(v) == 1:
+                                for i, vi in enumerate(v):
+                                    logging.scalar(f"layer[{idx}]/kernel.{k}[{i}]", vi)
+                # likelihood parameters
+                likelihood_components = _merge_leaf_components(
+                    leaf_components(self.model_gpflux.likelihood_layer.likelihood)
+                )
+                for k, v in likelihood_components.items():
+                    if v.trainable:
+                        logging.scalar(f"likelihood/{k}", v)
+
+                # losses and metrics
+                for k, v in self.model_keras.history.history.items():
+                    logging.histogram(f"{k}/epoch", lambda: v)
+                    logging.scalar(f"{k}/final", lambda: v[-1])
+                    logging.scalar(f"{k}/diff", lambda: v[0] - v[-1])
+                # training data based diagnostics
+                if dataset:
+                    predict = self.predict(dataset.query_points)
+                    # training accuracy
+                    diffs = dataset.observations - predict[0]
+                    z_residuals = diffs / tf.math.sqrt(predict[1])
+                    logging.histogram("accuracy/absolute_errors", tf.math.abs(diffs))
+                    logging.histogram("accuracy/z_residuals", z_residuals)
+                    logging.scalar(
+                        "accuracy/root_mean_square_error", tf.math.sqrt(tf.reduce_mean(diffs ** 2))
+                    )
+                    logging.scalar(
+                        "accuracy/mean_absolute_error", tf.reduce_mean(tf.math.abs(diffs))
+                    )
+                    logging.scalar("accuracy/z_residuals_std", tf.math.reduce_std(z_residuals))
+                    # training variance
+                    variance_error = predict[1] - diffs ** 2
+                    logging.histogram("variance/predict_variance", predict[1])
+                    logging.histogram("variance/variance_error", variance_error)
+                    logging.scalar("variance/predict_variance_mean", tf.reduce_mean(predict[1]))
+                    logging.scalar(
+                        "variance/root_mean_variance_error",
+                        tf.math.sqrt(tf.reduce_mean(variance_error ** 2)),
+                    )
+                    # data stats
+                    empirical_variance = tf.math.reduce_variance(dataset.observations)
+                    logging.scalar("variance/empirical", empirical_variance)
